@@ -1,23 +1,44 @@
 import os
 from logging import getLogger
 from pathlib import Path, PurePath
-from typing import List, Optional
+from typing import Dict, List, Optional
 from urllib.parse import urlparse
 
 import mlflow
+import mlflow.tracking.request_header.registry as mtrr  # necessary to access the global variable '_request_header_provider_registry' of the namespace
 from kedro.framework.context import KedroContext
+from kedro.utils import load_obj
 from mlflow.entities import Experiment
 from mlflow.tracking.client import MlflowClient
+from mlflow.tracking.request_header.abstract_request_header_provider import (
+    RequestHeaderProvider,
+)
 from pydantic import BaseModel, PrivateAttr, StrictBool
 from typing_extensions import Literal
 
 LOGGER = getLogger(__name__)
 
 
+class RequestHeaderProviderOptions(BaseModel):
+    # mutable default is ok for pydantic : https://stackoverflow.com/questions/63793662/how-to-give-a-pydantic-list-field-a-default-value
+    type: Optional[str] = None
+    pass_context: bool = False
+    init_kwargs: Dict[str, str] = {}
+
+    class Config:
+        extra = "forbid"
+        # necessary to check instance of RequestHeaderProvider:
+        arbitrary_types_allowed = "allowed"
+
+
 class MlflowServerOptions(BaseModel):
     # mutable default is ok for pydantic : https://stackoverflow.com/questions/63793662/how-to-give-a-pydantic-list-field-a-default-value
     mlflow_tracking_uri: Optional[str] = None
+    mlflow_registry_uri: Optional[str] = None
     credentials: Optional[str] = None
+    request_header_provider: RequestHeaderProviderOptions = (
+        RequestHeaderProviderOptions()
+    )
     _mlflow_client: MlflowClient = PrivateAttr()
 
     class Config:
@@ -104,23 +125,68 @@ class KedroMlflowConfig(BaseModel):
     def setup(self, context):
         """Setup all the mlflow configuration"""
 
-        self.server.mlflow_tracking_uri = _validate_mlflow_tracking_uri(
-            project_path=context.project_path, uri=self.server.mlflow_tracking_uri
+        # Manage the tracking uri
+        mlflow_tracking_uri = self.server.mlflow_tracking_uri
+        if mlflow_tracking_uri is None:
+            # do not use mlflow.get_tracking_uri() because if there is no env var,
+            # it resolves to 'Path.cwd() / "mlruns"'
+            # but we want 'project_path / "mlruns"'
+            mlflow_tracking_uri = os.environ.get("MLFLOW_TRACKING_URI", "mlruns")
+
+        self.server.mlflow_tracking_uri = _validate_uri(
+            project_path=context.project_path, uri=mlflow_tracking_uri
         )
 
+        # Manage the registry uri: if None, it will use the tracking
+        if self.server.mlflow_registry_uri is not None:
+            self.server.mlflow_registry_uri = _validate_uri(
+                project_path=context.project_path, uri=self.server.mlflow_registry_uri
+            )
+
         # init after validating the uri, else mlflow creates a mlruns folder at the root
         self.server._mlflow_client = MlflowClient(
-            tracking_uri=self.server.mlflow_tracking_uri
+            tracking_uri=self.server.mlflow_tracking_uri,
+            registry_uri=self.server.mlflow_registry_uri,
         )
 
         self._export_credentials(context)
 
+        self._register_request_header_provider(context)
         # we set the configuration now: it takes priority
         # if it has already be set in export_credentials
         mlflow.set_tracking_uri(self.server.mlflow_tracking_uri)
+        mlflow.set_registry_uri(self.server.mlflow_registry_uri)
 
         self._set_experiment()
 
+    def _register_request_header_provider(self, context: KedroContext):
+        # this is a specific trick to deal with expiring tokens for authentication, see https://github.com/Galileo-Galilei/kedro-mlflow/issues/357
+        if self.server.request_header_provider.type is not None:
+            # it type is none, there is nothing to register
+            request_header_provider_class = load_obj(
+                self.server.request_header_provider.type
+            )
+
+            if not issubclass(request_header_provider_class, RequestHeaderProvider):
+                raise ValueError(
+                    f"request_header_provider.type='{self.server.request_header_provider.type}' should be a sublass of 'mlflow.tracking.request_header.abstract_request_header_provider', got {request_header_provider_class} instead."
+                )
+
+            # we allow the user to instantiate the class with parameters, including the kedro context for advanded retrieval of kedro objects
+            init_kwargs = (
+                {
+                    **self.server.request_header_provider.init_kwargs,
+                    "kedro_context": context,
+                }
+                if self.server.request_header_provider.pass_context
+                else self.server.request_header_provider.init_kwargs
+            )
+
+            # the "register" method because expects a callable class with no arguments so we tricked it with a lambda
+            mtrr._request_header_provider_registry.register(
+                lambda: request_header_provider_class(**init_kwargs)
+            )
+
     def _export_credentials(self, context: KedroContext):
         conf_creds = context._get_config_credentials()
         mlflow_creds = conf_creds.get(self.server.credentials, {})
@@ -134,7 +200,6 @@ class KedroMlflowConfig(BaseModel):
         Returns:
             mlflow.entities.Experiment -- [description]
         """
-
         # we retrieve the experiment manually to check if it exsits
         mlflow_experiment = self.server._mlflow_client.get_experiment_by_name(
             name=self.tracking.experiment.name
@@ -164,7 +229,7 @@ class KedroMlflowConfig(BaseModel):
         )
 
 
-def _validate_mlflow_tracking_uri(project_path: str, uri: Optional[str]) -> str:
+def _validate_uri(project_path: str, uri: Optional[str]) -> str:
     """Format the uri provided to match mlflow expectations.
 
     Arguments:
@@ -174,15 +239,9 @@ def _validate_mlflow_tracking_uri(project_path: str, uri: Optional[str]) -> str:
         str -- A valid mlflow_tracking_uri
     """
 
-    # this is a special reserved keyword for mlflow which should not be converted to a path
-    # se: https://mlflow.org/docs/latest/tracking.html#where-runs-are-recorded
-    if uri is None:
-        # do not use mlflow.get_tracking_uri() because if there is no env var,
-        # it resolves to 'Path.cwd() / "mlruns"'
-        # but we want 'project_path / "mlruns"'
-        uri = os.environ.get("MLFLOW_TRACKING_URI", "mlruns")
-
     if uri == "databricks":
+        # "databricks" is a special reserved keyword for mlflow which should not be converted to a path
+        # see: https://mlflow.org/docs/latest/tracking.html#where-runs-are-recorded
         return uri
 
     # if no tracking uri is provided, we register the runs locally at the root of the project
@@ -200,7 +259,7 @@ def _validate_mlflow_tracking_uri(project_path: str, uri: Optional[str]) -> str:
             # See : https://discuss.python.org/t/pathlib-absolute-vs-resolve/2573/6
             valid_uri = (Path(project_path) / uri).as_uri()
             LOGGER.info(
-                f"The 'mlflow_tracking_uri' key in mlflow.yml is relative ('server.mlflow_tracking_uri = {uri}'). It is converted to a valid uri: '{valid_uri}'"
+                f"The 'tracking_uri' key in mlflow.yml is relative ('server.mlflow_(tracking|registry)_uri = {uri}'). It is converted to a valid uri: '{valid_uri}'"
             )
         else:
             # else assume it is an uri
